/*
 * Copyright (c) 2010-2015 TradertoolsFX
 *  
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:

 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.

 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
#property copyright "Copyright (c) 2010-2015 TradertoolsFX"
#property link      "http://www.tradertools-fx.com"
#property description "This content is released under the terms of the MIT license (http://opensource.org/licenses/mit-license.html)."
#property version "4.00"
#property strict
#property indicator_chart_window

#include <RetracementDrawer.mqh>

// User Parameters
input string FiboLevels = "0.0, 0.236, 0.382, 0.50, 0.618, 0.786, 1.0";
input string _desc1 = "===Enter parameters for fibonacci calculation below===";
input int LookbackBars = 0; // If 0, uses visible bars
input int BarShift = 0;
input bool WaitForCloseToUpdate = false;
input string _desc2 = "===Enter retraced/unretraced zone options below===";
input bool ShowRetracedZone = true;
input color RetracedZoneColor = Green;
input bool ShowUnretracedZone = true;
input color UnretracedZoneColor = Red;
input string _desc3 = "===Other parameters===";
input string FiboObjPrefix = "AutoFibo_";

RetracementDrawer *g_drawer;
bool g_redraw;
int g_end, g_start;

int init() 
{
    string prefix = StringConcatenate(FiboObjPrefix, TimeLocal(), "_");          
    double levels[];
    int num_levels = str_explode_double(FiboLevels, levels, ',');
    
    g_drawer = new RetracementDrawer(levels,
                                     num_levels, 
                                     prefix,
                                     RetracedZoneColor,
                                     UnretracedZoneColor);

    /* Use timer rather than start because if no ticks come in,
     * the chart objects will not respond to zoom. */
    EventSetTimer(1);                                                                                                                       
    return 0;
}

int deinit() 
{
    EventKillTimer();
    delete g_drawer;
    return 0;
}

void OnTimer()
{
    draw();
}

int start()
{
    return 0;
}

int draw() 
{   
    set_drawing_vars();
    
    // If drawing in real time or this is a new bar 
    if (!(WaitForCloseToUpdate || BarShift) || 
            is_new_bar()) {
        g_drawer.set_range(BarShift, g_end);
        
        draw_fibo();
        
        if (ShowRetracedZone) {
            draw_retraced_zone();
        } 
        
        if (ShowUnretracedZone) {
            draw_unretraced_zone();
        } 
    }   
   
    return 0;
}

/* Draw or update fibonacci retracement. */
void draw_fibo()
{
    int res = g_drawer.draw_fibo(g_redraw);
    if (res == -1) { // Failure
        Print("Failed to draw fibo");
    } else if (res == -2) { // Already exists
        if (g_drawer.update_fibo() != 0) { // Failure
            Print("Failed to update fibo");
        }
    }
}

/* Draw or update retraced zone. */
void draw_retraced_zone()
{    
    int res = g_drawer.draw_retraced_zone(g_redraw);
    if (res == -1) { // Failure
        Print("Failed to draw retraced zone");
    } else if (res == -2) { // Already exists
        if (g_drawer.update_retraced_zone() != 0) { // Failure
            Print("Failed to update retraced zone");
        }
    }    
}

/* Draw or update unretraced zone. */
void draw_unretraced_zone()
{
    int res = g_drawer.draw_unretraced_zone(g_redraw);
    if (res == -1) { // Failure
        Print("Failed to draw unretraced zone");
    } else if (res == -2) { // Already exists
        if (g_drawer.update_unretraced_zone() != 0) { // Failure
            Print("Failed to update unretraced zone");
        }
    }
}

/* Set globals that control drawing. */
void set_drawing_vars()
{
    int tmp;
    
    if (LookbackBars) {
        g_end = LookbackBars;
        g_redraw = false;
    } else {
        tmp = WindowFirstVisibleBar();
        g_redraw = (tmp != g_end);
        g_end = tmp;
    }
    
    g_start = BarShift;
}

bool is_new_bar() 
{
    static datetime open = 0;
    if (open != Time[0]) {
        open = Time[0];
        return true;
    }
    return false;
}

int str_explode_double(string list, 
                       double& out_array[], 
                       ushort delimiter=',') 
{
    string double_str[];
    int n = StringSplit(list, delimiter, double_str);
    
    if (ArrayResize(out_array, n) == -1) {
        Print(__FUNCTION__, " failed to resize explode_array!");
        return -1;
    }
    
    for (int i = 0; i < n; i++) {
        out_array[i] = StringToDouble(double_str[i]);
    }
    
    return n;
} 